import * as tslib_1 from "tslib";
import * as React from "react";
import { useSpring, animated } from "react-spring";
import { usePrevious } from "./use-previous";
import { useGestureResponder } from "react-gesture-responder";
import { Pane } from "./Pane";
import useScrollLock from "use-scroll-lock";
/**
 * React-Cube
 *
 * A 3d rotated cube which supports gestures
 * and an infinite number of views (i.e., it supports
 * windowing). It's inspired by the instagram story
 * inteface.
 */
export function Cube(_a) {
    var _b = _a.hasNext, hasNext = _b === void 0 ? function () { return true; } : _b, renderItem = _a.renderItem, _c = _a.width, width = _c === void 0 ? 200 : _c, _d = _a.height, height = _d === void 0 ? 600 : _d, _e = _a.perspective, perspective = _e === void 0 ? 1200 : _e, paneStyle = _a.paneStyle, _f = _a.scaleRange, scaleRange = _f === void 0 ? [1, 0.95] : _f, _g = _a.lockScrolling, lockScrolling = _g === void 0 ? false : _g;
    var _h = useSpring(function () { return ({ rotateY: 0 }); }), props = _h[0], set = _h[1];
    var _j = React.useState(0), index = _j[0], setIndex = _j[1];
    var prevIndex = usePrevious(index);
    var _k = React.useState([0, 1, 2, -1]), indexesToRender = _k[0], setIndexesToRender = _k[1];
    var currentActivePane = index % 4;
    var _l = React.useState(false), animating = _l[0], setAnimating = _l[1];
    // lock body scrolling when gesturing or animating
    useScrollLock(animating || lockScrolling);
    /**
     * Animate our cube into position when our active
     * index changes
     */
    function onRest() {
        setAnimating(false);
    }
    React.useEffect(function () {
        setAnimating(true);
        set({ rotateY: index * 90 * -1, immediate: false, onRest: onRest });
    }, [onRest, index, set]);
    /**
     * On drag end, determine the index to show
     */
    var onEnd = React.useCallback(function (_a) {
        var delta = _a.delta, velocity = _a.velocity;
        if (velocity > 0.12) {
            setIndex(delta[0] < 0 ? index + 1 : index - 1);
            return;
        }
        // next
        if (delta[0] < -(width / 2)) {
            setIndex(index + 1);
            // prev
        }
        else if (delta[0] > width / 2) {
            setIndex(index - 1);
            // current
        }
        else {
            set({ rotateY: index * 90 * -1, onRest: function () { }, immediate: false });
        }
    }, [index, width]);
    /**
     * Support basic drag gestures
     */
    var bind = useGestureResponder({
        onStartShouldSet: function () { return false; },
        onMoveShouldSet: function (_a) {
            var direction = _a.direction;
            // beginning of stack and swiping left
            if (index === 0 && direction[0] > 0) {
                return false;
            }
            // end of stack and swiping right
            if (direction[0] < 0 && !hasNext(index)) {
                return false;
            }
            // swiping horizontally
            return Math.abs(direction[0]) > Math.abs(direction[1]);
        },
        onRelease: onEnd,
        onTerminate: onEnd,
        onMove: function (_a) {
            var delta = _a.delta;
            if (!animating) {
                setAnimating(true);
            }
            var currentRotate = index * 90 * -1;
            var a = [0, width];
            var b = [currentRotate, currentRotate + 90];
            var x = delta[0];
            var o = a[1] - a[0];
            var n = b[1] - b[0];
            var v = ((x - a[0]) * n) / o + b[0];
            if (v > currentRotate + 90) {
                v = currentRotate + 90;
            }
            else if (v < currentRotate - 90) {
                v = currentRotate - 90;
            }
            set({
                rotateY: v,
                immediate: true,
                onRest: function () { }
            });
        }
    }).bind;
    /**
     * Handle rendering of individual panes.
     */
    React.useEffect(function () {
        // todo: usecallback style updates
        if (prevIndex && prevIndex !== index) {
            var movingForward = index > prevIndex;
            var indexes = indexesToRender.slice();
            if (movingForward) {
                var minIndex = indexes.indexOf(Math.min.apply(Math, indexes));
                indexes[minIndex] = Math.max.apply(Math, indexes) + 1;
            }
            else {
                var maxIndex = indexes.indexOf(Math.max.apply(Math, indexes));
                var next = Math.min.apply(Math, indexes) - 1;
                if (next > -1) {
                    indexes[maxIndex] = Math.min.apply(Math, indexes) - 1;
                }
            }
            setIndexesToRender(indexes);
        }
    }, [prevIndex, indexesToRender, index]);
    // todo: also alter scale slightly on rotate
    return (React.createElement("div", tslib_1.__assign({ className: "Cube", style: {
            width: width + "px",
            height: height + "px",
            perspective: perspective + "px"
        } }, bind),
        React.createElement(animated.div, { className: "Cube__animated-container", style: {
                width: "100%",
                height: "100%",
                position: "relative",
                backfaceVisibility: "visible",
                transformStyle: "preserve-3d",
                transform: props.rotateY.to(function (x) {
                    return "translateZ(-" + width / 2 + "px) rotateY(" + x + "deg) scale(" + getScale(x, scaleRange) + ")";
                })
            } },
            React.createElement(Pane, { tabIndex: -1, width: width, height: height, rotate: -90, active: currentActivePane === 3, style: paneStyle }, renderItem(indexesToRender[3], currentActivePane === 3 && !animating, props.rotateY)),
            React.createElement(Pane, { tabIndex: -1, width: width, height: height, rotate: 0, active: currentActivePane === 0, style: paneStyle }, renderItem(indexesToRender[0], currentActivePane === 0 && !animating, props.rotateY)),
            React.createElement(Pane, { tabIndex: -1, width: width, height: height, rotate: 90, active: currentActivePane === 1, style: paneStyle }, renderItem(indexesToRender[1], currentActivePane === 1 && !animating, props.rotateY)),
            React.createElement(Pane, { tabIndex: -1, width: width, height: height, rotate: -180, active: currentActivePane === 2, style: paneStyle }, renderItem(indexesToRender[2], currentActivePane === 2 && !animating, props.rotateY)))));
}
function getScale(x, scaleRange) {
    var diff = Math.abs(x) % 90;
    if (!diff) {
        return 1;
    }
    var a = diff > 45 ? [90, 45] : [45, 0];
    var b = diff > 45 ? scaleRange : [scaleRange[1], scaleRange[0]];
    var o = a[1] - a[0];
    var n = b[1] - b[0];
    var v = ((diff - a[0]) * n) / o + b[0];
    return v;
}
